function [SE,se,iter] = Consistent_Wiener_Filter(x,VS,VN,opts)

% CONSISTENT_WIENER_FILTER Wiener filtering with STFT consistency constraint
%
% [SE,se,iter] = Consistent_Wiener_Filter(x,VS,VN,opts)
%
% Inputs:
% x:  time-domain mixture signal (length should match an nbin x nfram STFT)
% VS: nbin x nfram STFT-domain source variance
% VN: nbin x nfram STFT-domain noise variance
% opts: optional parameters
%      type:   algorithm type (default: 'penalty')
%              'equality': Algorithm 1 of the paper, using equality
%                          constraint
%              'penalty':  Algorithm 2 of the paper, using penalty
%                          constraint
%      gamma:  weight of the STFT consistency penalty (default: 1e5)
%      hop:    STFT hopsize (default: half the frame length)
%
% Outputs:
% SE: nbin x nfram STFT-domain estimated source signal
% se: time-domain estimated source signal
% iter: number of iterations

% If you use this code, please cite:
% Le Roux, Jonathan and Vincent, Emmanuel, "Consistent Wiener Filtering 
% for Audio Source Separation," IEEE Signal Processing Letters, vol. 20, 
% no. 3, pp. 217--220, March 2013.
%   @article{LeRoux2013mar,
%     author = {{Le Roux}, J. and Vincent, E.},
%     title = {Consistent {W}iener Filtering for Audio Source Separation},
%     journal = {IEEE Signal Processing Letters},
%     volume = {20},
%     number = {3},
%     pages = {217--220},
%     month = mar,
%     year = 2013
%   }

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     Copyright (C) 2013  Emmanuel Vincent and Jonathan Le Roux
%
%     Patent Applied For in Japan: JP2011170190 
%
%     Commercial use of this software may be subject to limitations.
%
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Parse the input parameters

nsampl = length(x);
[nbin, nfram] = size(VS); 
wlen = 2*(nbin-1);  % VS and VN assumed to contain only DC to Nyquist

if ~exist('opts','var'), opts = struct; end
type   = parseopt(opts,'type','penalty');
gamma  = parseopt(opts,'gamma',1e5);
hop    = parseopt(opts,'hop',wlen/2);


%% Normalize and limit dynamic range

% mean -24 dB to avoid clipping
renorm = .0631/sqrt(sum(x.^2)/nsampl);
x  = renorm    * x; 
VS = renorm^2  * VS;
VN = renorm^2  * VN;

X  = stft(x, wlen, hop);
if nfram ~= size(X,2)
   error('The size of the variances does not match that of the signal.'); 
end

% limit the dynamic range to 90 dB
Vmin = 1e-9*max(max(VS(:)),max(VN(:)));
VS = VS+Vmin;
VN = VN+Vmin;

%% Consistent Wiener Filtering
switch type
    case 'equality'
        fprintf(1,'Computing Consistent Wiener Filter using Equality Constraint\n');
        [SE,iter] = conswiener_equality(X,VS,VN,nsampl,hop);
    case 'penalty'
        fprintf(1,'Computing Consistent Wiener Filter using Penalty Constraint with Gamma=%.4g\n',gamma);
        [SE,iter] = conswiener_penalty(X,VS,VN,gamma,nsampl,hop);
end
se = 1/renorm * istft(SE,hop,nsampl);
end

function [SE,iter] = conswiener_equality(X,VS,VN,nsampl,hop)

% CONSWIENER_EQUALITY Wiener filtering with STFT consistency constraint
%
% SE = conswiener_equality(X,VS,VN,nsampl,hop)
%
% Inputs:
% X: nbin x nfram STFT-domain mixture signal
% VS: nbin x nfram STFT-domain source variance
% VN: nbin x nfram STFT-domain noise variance
% gamma: weight of the STFT consistency penalty
% nsampl: number of samples of the original time-domain signal
% hop: STFT hopsize
%
% Outputs:
% SE: nbin x nfram STFT-domain estimated source signal
% 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) 2013 Emmanuel Vincent and Jonathan Le Roux
%
% Patent Applied For in Japan: JP2011170190 
%
% Commercial use of this software may be subject to limitations.
%
% This software is distributed under the terms of the GNU Public License
% version 3 (http://www.gnu.org/licenses/gpl.txt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

nbin=size(X,1);
wlen=2*(nbin-1);

%%% Unconstrained Wiener filter %%%
mu=VS./(VS+VN).*X;
Lambda=1./VS+1./VN;

%%% Conjugate gradient %%%
se=istft(mu,hop,nsampl);
SE=stft(se,wlen,hop);
b=istft(Lambda.*mu,hop,nsampl);
r=b-istft(Lambda.*SE,hop,nsampl);
invM=1./Lambda;
R=stft(r,wlen,hop);
z=istft(invM.*R,hop,nsampl);
p=z;
rsold=sum(sum(r.*z));
iter=0;
converged=false;
while ~converged,
    iter=iter+1;
    P=stft(p,wlen,hop);
    Ap=istft(Lambda.*P,hop,nsampl);
    alpha=rsold/sum(sum(p.*Ap)+realmin);
    se=se+alpha*p;
    converged=(alpha^2*sum(sum(p.*p)) < 1e-6*sum(sum(se.*se)));
    r=r-alpha*Ap;
    R=stft(r,wlen,hop);
    z=istft(invM.*R,hop,nsampl);
    rsnew=sum(sum(r.*z));
    beta=rsnew/(rsold+realmin);
    p=z+beta*p;
    rsold=rsnew;
end
SE=stft(se,wlen,hop);

end

function [SE,iter] = conswiener_penalty(X,VS,VN,gamma,nsampl,hop)

% CONSWIENER_PENALTY Wiener filtering with STFT consistency penalty
%
% SE=conswiener_penalty(X,VS,VN,gamma,nsampl,hop)
%
% Inputs:
% X: nbin x nfram STFT-domain mixture signal
% VS: nbin x nfram STFT-domain source variance
% VN: nbin x nfram STFT-domain noise variance
% gamma: weight of the STFT consistency penalty
% nsampl: number of samples of the original time-domain signal
% hop: STFT hopsize
%
% Outputs:
% SE: nbin x nfram STFT-domain estimated source signal
% 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright (C) 2013 Emmanuel Vincent and Jonathan Le Roux
%
% Patent Applied For in Japan: JP2011170190 
%
% Commercial use of this software may be subject to limitations.
%
% This software is distributed under the terms of the GNU Public License
% version 3 (http://www.gnu.org/licenses/gpl.txt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

[nbin,nfram]=size(X);
wlen=2*(nbin-1);

%%% Unconstrained Wiener filter %%%
SE=VS./(VS+VN).*X;
Lambda=1./VS+1./VN;

%%% Conjugate gradient %%%
invM=1./(Lambda+gamma*(wlen-hop)/wlen);
wei=repmat([1; 2*ones(nbin-2,1); 1],[1 nfram]);
se=istft(SE,hop,nsampl);
FSE=SE-stft(se,wlen,hop);
r=-gamma*FSE;
z=invM.*r;
P=z;
rsold=real(sum(sum(wei.*conj(r).*z)));
iter=0;
converged=false;
while ~converged,
    iter=iter+1;
    p=istft(P,hop,nsampl);
    FP=P-stft(p,wlen,hop);
    AP=Lambda.*P+gamma*FP;
    alpha=rsold/real(sum(sum(wei.*conj(P).*AP))+realmin);
    SE=SE+alpha*P;
    converged=(sum(sum(alpha^2*real(P.*conj(P)))) < 1e-6*sum(sum(real(SE.*conj(SE)))));
    r=r-alpha*AP;
    z=invM.*r;
    rsnew=real(sum(sum(wei.*conj(r).*z)));
    beta=rsnew/(rsold+realmin);
    P=z+beta*P;
    rsold=rsnew;
end

end

function var = parseopt(opts, varname, default)
    if isfield(opts, varname)
        var = opts.(varname);
    else
        var = default;
    end
end